-- About ALMA_NCIP_BORROWING_RENEWAL 1.0
-- Author: Kurt Munson, Northwestern University, kmunson@northwestern.edu
-- This server addon updates the due date in Alma from ILLiad using the RenewItem NCIP message.
-- The addon looks for request in an "Awaiting Alma Renewal Updating" queue, send the message 
-- using the value in the DateDue field of the transaction and routes the request to the Checked Out to Customer queue.

Local Settings = {};
Settings.NCIP_Responder_URL = GetSetting("NCIP_Responder_URL");
Settings.renewItem_from_uniqueAgency_value = GetSetting("renewItem_from_uniqueAgency_value");
Settings.ApplicationProfileType = GetSetting("ApplicationProfileType");
Settings.RenewItemSearchQueue = GetSetting("RenewItemSearchQueue");
Settings.RenewItemSuccessQueue = GetSetting("RenewItemSuccessQueue");
Settings.RenewItemFailQueue = GetSetting("RenewItemFailQueue");

local isCurrentlyProcessing = false;
local client = nil;

-- Assembly Loading and Type Importation
luanet.load_assembly("System");
local Types = {};
Types["WebClient"] = luanet.import_type("System.Net.WebClient");
Types["StreamReader"] = luanet.import_type("System.IO.StreamReader");

function Init()
	LogDebug("Initializing Alma NCIP Borrowing Renewal Addon");
	RegisterSystemEventHandler("SystemTimerElapsed", "TimerElapsed");
end

function TimerElapsed(eventArgs)
	LogDebug("Processing Alma NCIP Borrowing Renewal Items");
	if not isCurrentlyProcessing then
		isCurrentlyProcessing = true;

		-- Process Items
		local success, err = pcall(ProcessItems);
		if not success then
			LogDebug("There was a fatal error processing the items.")
			OnError(err);
		end

		isCurrentlyProcessing = false;
	else
		LogDebug("Still processing Alma NCIP Borrowing Renewal Items");
	end
end

function ProcessItems()
	ProcessDataContexts("TransactionStatus", Settings.RenewItemSearchQueue, "HandleContextProcessing");
end

function HandleContextProcessing()
	local transactionNumber = GetFieldValue("Transaction", "TransactionNumber");
	local RequestType = GetFieldValue("Transaction", "RequestType");
	LogDebug("Handling context processing for transaction: " .. transactionNumber);

	if RequestType == "Loan" then
	
	RenewItem()
	end

end


function RenewItem()
	LogDebug("Creating url");
	local user = GetFieldValue("Transaction", "Username");
	local transactionNumber = GetFieldValue("Transaction", "TransactionNumber");
	local dr = tostring(GetFieldValue("Transaction", "DueDate"));
	local df = string.match(dr, "%d+\/%d+\/%d+");
	local mn, dy, yr = string.match(df, "(%d+)/(%d+)/(%d+)");
	local mnt = string.format("%02d",mn);
	local dya = string.format("%02d",dy);
	local url = Settings.NCIP_Responder_URL;
	LogDebug(url);
	local body = [[<?xml version="1.0" encoding="ISO-8859-1"?>
	<NCIPMessage version="http://www.niso.org/schemas/ncip/v2_0/ncip_v2_0.xsd" xmlns="http://www.niso.org/2008/ncip">
	<RenewItem>
	<InitiationHeader>
	<FromAgencyId>
	<AgencyId>]] .. Settings.renewItem_from_uniqueAgency_value .. [[</AgencyId>
	</FromAgencyId> 
	<ToAgencyId>
	<AgencyId>]] .. Settings.renewItem_from_uniqueAgency_value .. [[</AgencyId>
	</ToAgencyId>
	<ApplicationProfileType>]] .. Settings.ApplicationProfileType .. [[</ApplicationProfileType>
	</InitiationHeader>
	<UserId>
	<UserIdentifierValue>]] .. user .. [[</UserIdentifierValue>
	</UserId>
	<ItemId>
	<ItemIdentifierValue>]] .. transactionNumber .. [[</ItemIdentifierValue>
	</ItemId>
	<DesiredDateDue>]] .. yr .. '-' .. mnt .. '-' .. dya .. [[</DesiredDateDue>
	</RenewItem>
	</NCIPMessage>
	]];

	-- LogDebug(body);
	
	LogDebug("Creating web client.");
	local webClient = Types["WebClient"]();
	webClient.Headers:Clear();
	webClient.Headers:Add("Content-Type", "text/xml; charset=UTF-8");
	LogDebug("Sending RenewItem message.");
	local responseString = webClient:UploadString(url, body);
	-- LogDebug(responseString);
	
	if string.find(responseString, "<ns1:DateDue>") then
	LogDebug("No Problems found in NCIP Response.")
	ExecuteCommand("Route", {transactionNumber, Settings.RenewItemSuccessQueue});
	ExecuteCommand("AddNote", {transactionNumber, "NCIP Response for RenewItem received successfully"});
    SaveDataSource("Transaction");
	
	else
	LogDebug("NCIP Error: ReRouting Transaction");
	ExecuteCommand("Route", {transactionNumber, Settings.RenewItemFailQueue});
	LogDebug("Adding Note to Transaction with NCIP Problem Error");
	ExecuteCommand("AddNote", {transactionNumber, responseString});
    SaveDataSource("Transaction");
	end

	end


